local _, T = ...
local L = T.L

local msq, msqGroups = nil, {}
if LibStub then
	msq = LibStub("Masque",true)
end
if msq then
	msqGroups = {
		FarmhandTools = msq:Group("Farmhand","Tools"),
		FarmhandSeeds = msq:Group("Farmhand","Seeds"),
	}
end

local Farmhand = CreateFrame("frame","Farmhand",UIParent)
Farmhand:SetDontSavePosition(true)
Farmhand:SetClampedToScreen(true)
Farmhand:RegisterEvent("ADDON_LOADED")
Farmhand:RegisterEvent("PLAYER_LOGIN")
Farmhand:RegisterEvent("PLAYER_ENTERING_WORLD")
local function Farmhand_EventHandler(self, event, ...)
	if event == "ADDON_LOADED" then
		local AddOn = ...
		if AddOn == "Farmhand" then
			self:Initialize()
			self:UnregisterEvent("ADDON_LOADED")
		end
	elseif event == "PLAYER_ENTERING_WORLD" then
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
		self:RegisterEvent("BAG_UPDATE")
		self:RegisterEvent("ZONE_CHANGED")
		self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
		self:RegisterEvent("ZONE_CHANGED_INDOORS")
		self:RegisterEvent("MERCHANT_SHOW")
		self:RegisterEvent("MERCHANT_CLOSED")
	elseif event == "PLAYER_LOGIN" or 
		   event == "ZONE_CHANGED" or 
		   event == "ZONE_CHANGED_NEW_AREA" or 
		   event == "ZONE_CHANGED_INDOORS" or
		   event == "BAG_UPDATE" then
		self:Update()
	elseif event == "MERCHANT_SHOW" or event == "MERCHANT_CLOSED" then
		if self:IsShown() then
			self.SellMode = event == "MERCHANT_SHOW"
			self:UpdateBar(self.seeds,true)
		end
	elseif event == "PLAYER_REGEN_ENABLED" then
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
		if self.Initialized then
			self:Update()
		else
			self:Initialize()
		end	
	end
end
Farmhand:SetScript("OnEvent",Farmhand_EventHandler)
Farmhand:SetMovable(true)
Farmhand.L = L

Farmhand.tools = CreateFrame("Frame","FarmhandTools",Farmhand)
Farmhand.tools:SetPoint("Bottom",Farmhand,"Bottom",0,7)
Farmhand.tools:SetScale(1.5)
Farmhand.tools.items = {
	{id=89815}, --Master Plow
	{id=89880}, --Dented Shovel
	{id=80513}, --Vintage Bug Sprayer
	{id=79104}, --Rusty Watering Can
	{id="scanner"}, --Plant Scanner
}

Farmhand.seeds = CreateFrame("Frame","FarmhandSeeds",Farmhand)
Farmhand.seeds:SetPoint("Top",Farmhand,"Top",0,-15)
Farmhand.seeds:SetScale(1)
Farmhand.seeds.items = {
	{id=79102}, --Green Cabbage Seeds
	{id=89328}, --Jade Squash Seeds
	{id=80590}, --Juicycrunch Carrot Seeds
	{id=80592}, --Mogu Pumpkin Seeds
	{id=80594}, --Pink Turnip Seeds
	{id=80593}, --Red Blossom Leek Seeds
	{id=80591}, --Scallion Seeds
	{id=89329}, --Striped Melon Seeds
	{id=80595}, --White Turnip Seeds
	{id=89326}, --Witchberry Seeds
	{id=85219}, --Ominous Seed
	{id=85216}, --Enigma Seed
	{id=85217}, --Magebulb Seed
	{id=89202}, --Raptorleaf Seed
	{id=85215}, --Snakeroot Seed
	{id=89233}, --Songbell Seed
	{id=89197}, --Windshear Cactus Seed
	{id=91860}, --Stormwind Portal Shard
	{id=91864}, --Ironforge Portal Shard
	{id=91865}, --Darnassus Portal Shard
	{id=91866}, --Exodar Portal Shard
	{id=91850}, --Orgrimmar Portal Shard
	{id=91862}, --Undercity Portal Shard
	{id=91861}, --Thunder Bluff Portal Shard
	{id=91863}, --Silvermoon Portal Shard
}

local lock = CreateFrame("CheckButton","FarmhandToolLock",FarmhandTools)
lock:SetBackdrop({bgFile="Interface\\Buttons\\LockButton-Border"})
lock:SetNormalTexture("Interface\\Buttons\\LockButton-Unlocked-Up")
lock:SetPushedTexture("Interface\\Buttons\\LockButton-Unlocked-Down")
lock:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
lock:SetCheckedTexture("Interface\\Buttons\\LockButton-Locked-Up")
lock:SetSize(16,16)
lock:SetPoint("Right",FarmhandTools,"Left",0,0)
RegisterStateDriver(lock,"visibility","[mod:shift] show; hide")
lock:SetScript("OnClick",function(self)
	if self:GetChecked() == 1 then
		FarmhandData.ToolsLocked = true
		print(L["Tools are now |cFF00FF00Locked|r and |cFF00FF00will not|r be dropped when you leave Halfhill."])
	else
		FarmhandData.ToolsLocked = false
		print(L["Tools are now |cFFFF0000Unlocked|r and |cFFFF0000will|r be dropped when you leave Halfhill."])
	end
end)
Farmhand.lock = lock

local f = CreateFrame("CheckButton","FarmhandScanMessagesOption",FarmhandTools,"OptionsSmallCheckButtonTemplate")
f:SetPoint("BottomLeft",FarmhandTools,"Right")
f:SetScale(.5)
_G["FarmhandScanMessagesOptionText"]:SetText(L["Print Messages"])
f:SetScript("OnClick",function(self)
	if self:GetChecked() == 1 then
		FarmhandData.PrintScannerMessages = true
	else
		FarmhandData.PrintScannerMessages = false
	end
end)
RegisterStateDriver(f,"visibility","[mod:shift] show; hide")
Farmhand.MessagesOption = f

f = CreateFrame("CheckButton","FarmhandScanSoundOption",FarmhandTools,"OptionsSmallCheckButtonTemplate")
f:SetPoint("TopLeft",FarmhandTools,"Right")
f:SetScale(.5)
_G["FarmhandScanSoundOptionText"]:SetText(L["Play Sounds"])
f:SetScript("OnClick",function(self)
	if self:GetChecked() == 1 then
		FarmhandData.PlayScannerSounds = true
	else
		FarmhandData.PlayScannerSounds = false
	end
end)
RegisterStateDriver(f,"visibility","[mod:shift] show; hide")
Farmhand.SoundOption = f

function Farmhand:Initialize()
	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end
	if FarmhandData == nil then
		FarmhandData = {
			X = 0,
			Y = -UIParent:GetHeight() / 4,
			ToolsLocked = false,
			PrintScannerMessages = true,
			PlayScannerSounds = true,
		}
	end

--	print("Intial X="..FarmhandData.X.." Y="..FarmhandData.Y)
	Farmhand:SetPoint("Center",UIParent,"Center",FarmhandData.X,FarmhandData.Y)
	
	self.lock:SetChecked(FarmhandData.ToolsLocked == true)
	Farmhand.MessagesOption:SetChecked(FarmhandData.PrintScannerMessages == true)
	Farmhand.SoundOption:SetChecked(FarmhandData.PlayScannerSounds == true)
	
	self:CreateButtons(self.tools)
	self:CreateButtons(self.seeds)

	self.Initialized = true
end

function Farmhand:CreateButtons(bar)
	for i,item in ipairs(bar.items) do
	
		if item.id == "scanner" then
			item.btn = CreateFrame("Button", "FarmhandScanButton", FarmhandTools, "SecureActionButtonTemplate")
			item.btn:SetSize(32,32)
			item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")
			item.btn.icon:SetTexture("Interface\\AddOns\\Farmhand\\RadarIcon.tga")
			item.btn.icon:SetAllPoints()
			item.btn.icon:Show()
			
			item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
			item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")

			item.btn:RegisterForClicks("AnyUp")
			item.btn:SetAttribute("type", "click")

			local CropStates = {
				{ CropNames = L["Occupied Soil"], Icon = 8 },
				{ CropNames = L["Stubborn Weed"], Icon = 4 },
				{ CropNames = L["AlluringCropNames"], Icon = 5 },
				{ CropNames = L["WildCropNames"], Icon = 3 },
				{ CropNames = L["TangledCropNames"], Icon = 7 },
				{ CropNames = L["ParchedCropNames"], Icon = 6 },
				{ CropNames = L["InfestedCropNames"], Icon = 4 },
				{ CropNames = L["WigglingCropNames"], Icon = 8 },
				{ CropNames = L["SmotheredCropNames"], Icon = 2 },
				{ CropNames = L["BurstingCropNames"], Icon = 2 },
				{ CropNames = L["RuntyCropNames"], Icon = 1 },
			}

			local NewMacroButton = function(Step, SubStep, MacroText)
				--print("Creating ".."FHSBE_"..Step.."_"..SubStep)
				local f = CreateFrame("Button", "FHSBE_"..Step.."_"..SubStep, FarmhandScanButton, "SecureActionButtonTemplate")
				f:SetSize(32,32)
				f:RegisterForClicks("AnyUp")
				f:SetAttribute("type", "macro")
				f:SetAttribute("macrotext", MacroText)
				return f
			end
			
			for Step, State in ipairs(CropStates) do

				local SubStep = 1
				local MacroText = "/cleartarget\n"
				local IconLine = "/run if UnitExists(\"target\") and GetRaidTargetIndex(\"target\") ~= "..State.Icon.." then SetRaidTarget(\"target\","..State.Icon..") end\n"

				local Lines = { strsplit("\n", gsub(State.CropNames,"\r","")) }
				for i,line in ipairs(Lines) do
					line = "/tar "..strtrim(line).."\n"
					if strlen(MacroText) + strlen(line) + strlen(IconLine) + strlen("/click FHSBE_9999_9999\n") > 1023 then
						MacroText = MacroText..format("/click FHSBE_%d_%d\n",Step,SubStep+1)
						NewMacroButton(Step, SubStep, MacroText)
						SubStep = SubStep + 1
						MacroText = ""
					end
					MacroText = MacroText..line
				end
				MacroText = MacroText..IconLine
				MacroText = MacroText..format("/click FHSBE_%d_%d\n",Step+1,1)
				local f = NewMacroButton(Step, SubStep, MacroText)
				f:SetScript("PreClick",Farmhand.CropScannerCheckResult)
				if Step == #CropStates then
					f:SetScript("PostClick",function()
						FarmhandScanButton.Finished = true
						Farmhand.CropScannerCheckResult()
					end)
				end
			end

			item.btn:SetAttribute("clickbutton", FHSBE_1_1)
			item.btn:SetScript("PreClick",Farmhand.CropScannerInitialize)
			
		else
			item.btn = CreateFrame("Button", bar:GetName().."Button"..i, bar, "SecureActionButtonTemplate")
			item.btn:SetSize(32,32)

			item.btn.icon = item.btn:CreateTexture(item.btn:GetName().."Icon","BACKGROUND")
			item.btn.icon:SetAllPoints()
			item.btn.icon:Show()

			item.btn.count = item.btn:CreateFontString(item.btn:GetName().."Count","ARTWORK","NumberFontNormal")
			item.btn.count:SetJustifyH("RIGHT")
			item.btn.count:SetPoint("BottomRight",-2,2)

			item.btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
			item.btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
			
			item.btn:RegisterForClicks("AnyUp")
		end

		item.btn:SetScript("OnEnter",Farmhand.ButtonOnEnter)
		item.btn:SetScript("OnLeave",Farmhand.ButtonOnLeave)
		item.btn:SetScript("OnMouseDown",Farmhand.ButtonOnMouseDown)
		item.btn:SetScript("OnMouseUp",Farmhand.ButtonOnMouseUp)
		item.btn:SetScript("OnHide",Farmhand.ButtonOnHide)
		
		if msqGroups[bar:GetName()] then
			msqGroups[bar:GetName()]:AddButton(item.btn)
		end

	end
end

function Farmhand.CropScannerInitialize()
	FarmhandScanButton.AllClear = true
	FarmhandScanButton.Finished = false
end

function Farmhand.CropScannerCheckResult()
	if UnitExists("target") then
		FarmhandScanButton.AllClear = false
		if FarmhandData.PrintScannerMessages then
			CropName = UnitName("target")
			Icon = ICON_LIST[GetRaidTargetIndex("target")] and ICON_LIST[GetRaidTargetIndex("target")].."0|t" or ""
			local msg = L["Crop Scanner found:"].." "..Icon.." "..CropName
			print(msg)
			--RaidNotice_AddMessage(RaidBossEmoteFrame,msg, ChatTypeInfo["RAID_BOSS_EMOTE"])
		end
	end
	if FarmhandScanButton.Finished then
		if FarmhandScanButton.AllClear then
			if FarmhandData.PrintScannerMessages then
				print(L["Crop Scanner finished."].." "..L["The crops are looking good!"])
				RaidNotice_AddMessage(RaidBossEmoteFrame,L["The crops are looking good!"], ChatTypeInfo["RAID_BOSS_EMOTE"])
			end
			if FarmhandData.PlayScannerSounds then
				PlaySound("QUESTCOMPLETED","SFX")
			end
		else
			if FarmhandData.PrintScannerMessages then
				RaidNotice_AddMessage(RaidBossEmoteFrame,L["Some crops need attention!"], ChatTypeInfo["RAID_BOSS_EMOTE"])
			end
			if FarmhandData.PlayScannerSounds then
				PlaySound("igQuestFailed","SFX")
			end
		end
	end
end

function Farmhand:ButtonOnEnter()
	if Farmhand.SellMode and self:GetParent():GetName() == "FarmhandSeeds" then
		ShowContainerSellCursor(self.bag,self.slot)
	end
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	if self:GetName() == "FarmhandScanButton" then
		GameTooltip:ClearLines()
		GameTooltip:AddLine(L["Crop Scanner"],0,255,0,false)
		GameTooltip:AddLine(L["Scan your farm for crops or soil that need attention."],255,255,255,true)
	else
		GameTooltip:SetBagItem(self.bag,self.slot)
	end
	GameTooltip:Show()
end

function Farmhand:ButtonOnLeave()
	if Farmhand.SellMode and self:GetParent():GetName() == "FarmhandSeeds" then
		ResetCursor()
	end
	GameTooltip_Hide()
end

function Farmhand:ButtonOnMouseDown(button)
	if IsShiftKeyDown() and button == "LeftButton" and not Farmhand.isMoving then
		_,_,_, Farmhand.InitialOffsetX, Farmhand.InitialOffsetY = Farmhand:GetPoint(1)
--		print("InitialOffsetX: "..Farmhand.InitialOffsetX.." InitialOffsetY: "..Farmhand.InitialOffsetY)
		Farmhand:StartMoving()
		_,_,_, Farmhand.PickupOffsetX, Farmhand.PickupOffsetY = Farmhand:GetPoint(1)
--		print("PickupOffsetX: "..Farmhand.PickupOffsetX.." PickupOffsetY: "..Farmhand.PickupOffsetY)
		Farmhand.isMoving = true
	elseif button == "RightButton" and Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		FarmhandData.X, FarmhandData.Y = 0, - UIParent:GetHeight() / 4
		Farmhand:ClearAllPoints()
		Farmhand:SetPoint("Center",UIParent,"Center",FarmhandData.X,FarmhandData.Y)
	end
end

function Farmhand:ButtonOnMouseUp(button)
	if button == "LeftButton" and Farmhand.isMoving then
		local _,_,_, DropOffsetX, DropOffsetY = Farmhand:GetPoint(1)
--		print("DropOffsetX: "..DropOffsetX.." DropOffsetY: "..DropOffsetY)
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		FarmhandData.X = DropOffsetX - Farmhand.PickupOffsetX + Farmhand.InitialOffsetX
		FarmhandData.Y = DropOffsetY - Farmhand.PickupOffsetY + Farmhand.InitialOffsetY
--		print("FinalOffsetX: "..FarmhandData.X.." FinalOffsetY: "..FarmhandData.Y)
		Farmhand:ClearAllPoints()
		Farmhand:SetPoint("Center",UIParent,FarmhandData.X,FarmhandData.Y)
   end
end

function Farmhand:ButtonOnHide()
	if Farmhand.isMoving then
		Farmhand:StopMovingOrSizing()
		Farmhand.isMoving = false
		Farmhand:ClearAllPoints()
		Farmhand:SetPoint("Center", UIParent, "Center", FarmhandData.X, FarmhandData.Y)
	end
end

function Farmhand:UpdateBar(bar, show)
	local last
	local shown = 0
	local ButtonSpacing = 6

	for _,item in ipairs(bar.items) do
		local ItemCount = GetItemCount(item.id)
		if ItemCount > 0 or item.id == "scanner" then
			if shown % 8 == 0 then
				local row = math.floor(shown/8)
				item.btn:SetPoint("TopLeft",bar,"TopLeft", ButtonSpacing/2, -(ButtonSpacing * .5 ) - row*item.btn:GetHeight() - row*ButtonSpacing)
				last = item.btn
			else
				item.btn:SetPoint("TopLeft",last,"TopRight",ButtonSpacing,0)
				last = item.btn
			end
			if IsConsumableItem(item.id) then
				local count = _G[item.btn:GetName().."Count"]
				if ItemCount > 999 then
					count:SetText("***")
				else
					count:SetText(ItemCount)
				end
			end
			local bag, slot = self:FindItemInBags(item.id)
			if bag and slot then
				local icon = GetContainerItemInfo(bag,slot)
				item.btn.bag = bag
				item.btn.slot = slot
				item.btn.icon:SetTexture(icon)
				if bar:GetName() == "FarmhandSeeds" then
					item.btn:SetAttribute("type", "macro")
					item.btn:SetAttribute("macrotext","/targetexact [noexists] "..L["Tilled Soil"].."\n/use "..bag.." "..slot)
				else
					item.btn:SetAttribute("type", "item")
					item.btn:SetAttribute("item", bag.." "..slot)
				end
			end
			item.btn:Show()
			shown = shown + 1
		else
			item.btn:Hide()
		end
	end

	if msqGroups[bar:GetName()] then msqGroups[bar:GetName()]:ReSkin() end
	
	if last ~= nil and show then
		bar:SetWidth(math.min(8,shown) * (last:GetWidth() + ButtonSpacing))
		bar:SetHeight(math.ceil(shown/8) * (last:GetHeight() + ButtonSpacing))
		bar:Show()
	else
		bar:Hide()
	end
end

function Farmhand:Update()

	if InCombatLockdown() then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
		return
	end

	local Zone, SubZone = GetZoneText(), GetSubZoneText()

	if Zone == "" and SubZone == "" then 
		return
	end

	if self.InHalfhill and SubZone ~= L["Sunsong Ranch"] and SubZone ~= L["The Halfhill Market"] and SubZone ~= L["Halfhill"] and Zone ~= L["Halfhill"] then
		if FarmhandData.ToolsLocked == true then
			print(L["Leaving Halfhill."].." "..L["Tools are Locked."])
		else
			ClearCursor()
			for i,item in ipairs(self.tools.items) do
				local bag, slot = self:FindItemInBags(item.id)
				if bag and slot then
					PickupContainerItem(bag,slot)
					if CursorHasItem() then
						local _, id, link = GetCursorInfo()
						if id == item.id then
							print(L["Leaving Halfhill."].." "..L["Dropping"].." "..link..".")
							DeleteCursorItem()
						end
					end
				end
			end
		end
	end

	self.InHalfhill = SubZone == L["Sunsong Ranch"] or SubZone == L["The Halfhill Market"] or SubZone == L["Halfhill"] or Zone == L["Halfhill"]
	
	if SubZone == L["Sunsong Ranch"] or SubZone == L["The Halfhill Market"] then 
		self:UpdateBar(self.tools,SubZone == L["Sunsong Ranch"])
		self:UpdateBar(self.seeds,true)
			
		local th = (self.tools:IsShown() and self.tools:GetHeight()*self.tools:GetScale() or 0)
		local sh = (self.seeds:IsShown() and self.seeds:GetHeight()*self.seeds:GetScale() or 0)
		self:SetHeight(24 + th + sh)
		
		local tw = (self.tools:IsShown() and self.tools:GetWidth()*self.tools:GetScale() or 0)
		local sw = (self.seeds:IsShown() and self.seeds:GetWidth()*self.seeds:GetScale() or 0)
		self:SetWidth(24 + math.max(tw,sw))
		
		self:Show()
	elseif self:IsShown() then
		self:Hide()
	end

end

function Farmhand:FindItemInBags(item)
	for container = 0, NUM_BAG_SLOTS do
		local numSlots = GetContainerNumSlots(container)
		if numSlots then
			for slot = 1, numSlots do
				if item == GetContainerItemID(container, slot) then
					return container, slot
				end
			end
		end
	end
end
